Class: Rake::Application

Inherits:
Object
  • Object
show all
Includes:
TaskManager, TraceOutput
Defined in:
lib/rake/application.rb

Overview

Rake main application object. When invoking rake from the command line, a Rake::Application object is created and run.

Constant Summary collapse

DEFAULT_RAKEFILES =
[
  "rakefile",
  "Rakefile",
  "rakefile.rb",
  "Rakefile.rb"
].freeze

Instance Attribute Summary collapse

Attributes included from TaskManager

#last_description

Instance Method Summary collapse

Methods included from TraceOutput

#trace_on

Methods included from TaskManager

#[], #clear, #create_rule, #current_scope, #define_task, #enhance_with_matching_rule, #generate_did_you_mean_suggestions, #generate_message_for_undefined_task, #in_namespace, #intern, #lookup, #resolve_args, #synthesize_file_task, #tasks, #tasks_in_scope

Constructor Details

#initializeApplication

Initialize a Rake::Application object.



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
# File 'lib/rake/application.rb', line 50

def initialize
  super
  @name = "rake"
  @rakefiles = DEFAULT_RAKEFILES.dup
  @rakefile = nil
  @pending_imports = []
  @imported = []
  @loaders = {}
  @default_loader = Rake::DefaultLoader.new
  @original_dir = Dir.pwd
  @top_level_tasks = []
  add_loader("rb", DefaultLoader.new)
  add_loader("rf", DefaultLoader.new)
  add_loader("rake", DefaultLoader.new)
  @tty_output = STDOUT.tty?
  @terminal_columns = ENV["RAKE_COLUMNS"].to_i

  set_default_options
end

Instance Attribute Details

#nameObject (readonly)

The name of the application (typically ‘rake’)



25
26
27
# File 'lib/rake/application.rb', line 25

def name
  @name
end

#original_dirObject (readonly)

The original directory where rake was invoked.



28
29
30
# File 'lib/rake/application.rb', line 28

def original_dir
  @original_dir
end

#rakefileObject (readonly)

Name of the actual rakefile used.



31
32
33
# File 'lib/rake/application.rb', line 31

def rakefile
  @rakefile
end

#terminal_columnsObject

Number of columns on the terminal



34
35
36
# File 'lib/rake/application.rb', line 34

def terminal_columns
  @terminal_columns
end

#top_level_tasksObject (readonly)

List of the top level task names (task names from the command line).



37
38
39
# File 'lib/rake/application.rb', line 37

def top_level_tasks
  @top_level_tasks
end

#tty_output=(value) ⇒ Object (writeonly)

Override the detected TTY output state (mostly for testing)



40
41
42
# File 'lib/rake/application.rb', line 40

def tty_output=(value)
  @tty_output = value
end

Instance Method Details

#add_import(fn) ⇒ Object

Add a file to the list of files to be imported.



793
794
795
# File 'lib/rake/application.rb', line 793

def add_import(fn) # :nodoc:
  @pending_imports << fn
end

#add_loader(ext, loader) ⇒ Object

Add a loader to handle imported files ending in the extension ext.



162
163
164
165
# File 'lib/rake/application.rb', line 162

def add_loader(ext, loader)
  ext = ".#{ext}" unless ext =~ /^\./
  @loaders[ext] = loader
end

#collect_command_line_tasks(args) ⇒ Object

Collect the list of tasks on the command line. If no tasks are given, return a list containing only the default task. Environmental assignments are processed at this time as well.

‘args` is the list of arguments to peruse to get the list of tasks. It should be the command line that was given to rake, less any recognised command-line options, which OptionParser.parse will have taken care of already.



774
775
776
777
778
779
780
781
782
783
784
# File 'lib/rake/application.rb', line 774

def collect_command_line_tasks(args) # :nodoc:
  @top_level_tasks = []
  args.each do |arg|
    if arg =~ /^(\w+)=(.*)$/m
      ENV[$1] = $2
    else
      @top_level_tasks << arg unless arg =~ /^-/
    end
  end
  @top_level_tasks.push(default_task_name) if @top_level_tasks.empty?
end

#default_task_nameObject

Default task name (“default”). (May be overridden by subclasses)



788
789
790
# File 'lib/rake/application.rb', line 788

def default_task_name # :nodoc:
  "default"
end

#deprecate(old_usage, new_usage, call_site) ⇒ Object

Warn about deprecated usage.

Example:

Rake.application.deprecate("import", "Rake.import", caller.first)


283
284
285
286
287
288
289
# File 'lib/rake/application.rb', line 283

def deprecate(old_usage, new_usage, call_site) # :nodoc:
  unless options.ignore_deprecate
    $stderr.puts "WARNING: '#{old_usage}' is deprecated.  " +
      "Please use '#{new_usage}' instead.\n" +
      "    at #{call_site}"
  end
end

#display_cause_details(ex) ⇒ Object

:nodoc:



245
246
247
248
249
250
# File 'lib/rake/application.rb', line 245

def display_cause_details(ex) # :nodoc:
  return if display_exception_details_seen.include? ex

  trace "\nCaused by:"
  display_exception_details(ex)
end

#display_error_message(ex) ⇒ Object

Display the error message that caused the exception.



229
230
231
232
233
234
235
# File 'lib/rake/application.rb', line 229

def display_error_message(ex) # :nodoc:
  trace "#{name} aborted!"
  display_exception_details(ex)
  trace "Tasks: #{ex.chain}" if has_chain?(ex)
  trace "(See full trace by running task with --trace)" unless
     options.backtrace
end

#display_exception_backtrace(ex) ⇒ Object

:nodoc:



270
271
272
273
274
275
276
# File 'lib/rake/application.rb', line 270

def display_exception_backtrace(ex) # :nodoc:
  if options.backtrace
    trace ex.backtrace.join("\n")
  else
    trace Backtrace.collapse(ex.backtrace).join("\n")
  end
end

#display_exception_details(ex) ⇒ Object

:nodoc:



237
238
239
240
241
242
243
# File 'lib/rake/application.rb', line 237

def display_exception_details(ex) # :nodoc:
  display_exception_details_seen << ex

  display_exception_message_details(ex)
  display_exception_backtrace(ex) if ex.backtrace
  display_cause_details(ex.cause) if has_cause?(ex)
end

#display_exception_details_seenObject

:nodoc:



252
253
254
# File 'lib/rake/application.rb', line 252

def display_exception_details_seen # :nodoc:
  Thread.current[:rake_display_exception_details_seen] ||= []
end

#display_exception_message_details(ex) ⇒ Object

:nodoc:



260
261
262
263
264
265
266
267
268
# File 'lib/rake/application.rb', line 260

def display_exception_message_details(ex) # :nodoc:
  if ex.instance_of?(RuntimeError)
    trace ex.message
  elsif ex.respond_to?(:detailed_message)
    trace "#{ex.class.name}: #{ex.detailed_message(highlight: false)}"
  else
    trace "#{ex.class.name}: #{ex.message}"
  end
end

#display_prerequisitesObject

Display the tasks and prerequisites



406
407
408
409
410
411
# File 'lib/rake/application.rb', line 406

def display_prerequisites # :nodoc:
  tasks.each do |t|
    puts "#{name} #{t.name}"
    t.prerequisites.each { |pre| puts "    #{pre}" }
  end
end

#display_tasks_and_commentsObject

Display the tasks and comments.



323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
# File 'lib/rake/application.rb', line 323

def display_tasks_and_comments # :nodoc:
  displayable_tasks = tasks.select { |t|
    (options.show_all_tasks || t.comment) &&
      t.name =~ options.show_task_pattern
  }
  case options.show_tasks
  when :tasks
    width = displayable_tasks.map { |t| t.name_with_args.length }.max || 10
    if truncate_output?
      max_column = terminal_width - name.size - width - 7
    else
      max_column = nil
    end

    displayable_tasks.each do |t|
      printf("#{name} %-#{width}s  # %s\n",
        t.name_with_args,
        max_column ? truncate(t.comment, max_column) : t.comment)
    end
  when :describe
    displayable_tasks.each do |t|
      puts "#{name} #{t.name_with_args}"
      comment = t.full_comment || ""
      comment.split("\n").each do |line|
        puts "    #{line}"
      end
      puts
    end
  when :lines
    displayable_tasks.each do |t|
      t.locations.each do |loc|
        printf "#{name} %-30s %s\n", t.name_with_args, loc
      end
    end
  else
    fail "Unknown show task mode: '#{options.show_tasks}'"
  end
end

#dynamic_widthObject

Calculate the dynamic width of the



374
375
376
# File 'lib/rake/application.rb', line 374

def dynamic_width # :nodoc:
  @dynamic_width ||= (dynamic_width_stty.nonzero? || dynamic_width_tput)
end

#dynamic_width_sttyObject

:nodoc:



378
379
380
# File 'lib/rake/application.rb', line 378

def dynamic_width_stty # :nodoc:
  %x{stty size 2>/dev/null}.split[1].to_i
end

#dynamic_width_tputObject

:nodoc:



382
383
384
# File 'lib/rake/application.rb', line 382

def dynamic_width_tput # :nodoc:
  %x{tput cols 2>/dev/null}.to_i
end

#exit_because_of_exception(ex) ⇒ Object

Exit the program because of an unhandled exception. (may be overridden by subclasses)



224
225
226
# File 'lib/rake/application.rb', line 224

def exit_because_of_exception(ex) # :nodoc:
  exit(false)
end

#find_rakefile_locationObject

:nodoc:



703
704
705
706
707
708
709
710
711
712
713
# File 'lib/rake/application.rb', line 703

def find_rakefile_location # :nodoc:
  here = Dir.pwd
  until (fn = have_rakefile)
    Dir.chdir("..")
    return nil if Dir.pwd == here || options.nosearch
    here = Dir.pwd
  end
  [fn, here]
ensure
  Dir.chdir(Rake.original_dir)
end

#handle_options(argv) ⇒ Object

Read and handle the command line options. Returns the command line arguments that we didn’t understand, which should (in theory) be just task names and env vars.



669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
684
685
# File 'lib/rake/application.rb', line 669

def handle_options(argv) # :nodoc:
  set_default_options

  OptionParser.new do |opts|
    opts.banner = "#{Rake.application.name} [-f rakefile] {options} targets..."
    opts.separator ""
    opts.separator "Options are ..."

    opts.on_tail("-h", "--help", "-H", "Display this help message.") do
      puts opts
      exit
    end

    standard_rake_options.each { |args| opts.on(*args) }
    opts.environment("RAKEOPT")
  end.parse(argv)
end

#has_cause?(ex) ⇒ Boolean

:nodoc:

Returns:

  • (Boolean)


256
257
258
# File 'lib/rake/application.rb', line 256

def has_cause?(ex) # :nodoc:
  ex.respond_to?(:cause) && ex.cause
end

#have_rakefileObject

True if one of the files in RAKEFILES is in the current directory. If a match is found, it is copied into @rakefile.



299
300
301
302
303
304
305
306
307
308
309
# File 'lib/rake/application.rb', line 299

def have_rakefile # :nodoc:
  @rakefiles.each do |fn|
    if File.exist?(fn)
      others = FileList.glob(fn, File::FNM_CASEFOLD)
      return others.size == 1 ? others.first : fn
    elsif fn == ""
      return fn
    end
  end
  return nil
end

#init(app_name = "rake", argv = ARGV) ⇒ Object

Initialize the command line parameters and app name.



89
90
91
92
93
94
95
96
97
98
99
100
101
# File 'lib/rake/application.rb', line 89

def init(app_name="rake", argv = ARGV)
  standard_exception_handling do
    @name = app_name
    begin
      args = handle_options argv
    rescue ArgumentError
      # Backward compatibility for capistrano
      args = handle_options
    end
    load_debug_at_stop_feature
    collect_command_line_tasks(args)
  end
end

#invoke_task(task_string) ⇒ Object

Invokes a task with arguments that are extracted from task_string



180
181
182
183
184
# File 'lib/rake/application.rb', line 180

def invoke_task(task_string) # :nodoc:
  name, args = parse_task_string(task_string)
  t = self[name]
  t.invoke(*args)
end

#load_importsObject

Load the pending list of imported files.



798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
# File 'lib/rake/application.rb', line 798

def load_imports # :nodoc:
  while fn = @pending_imports.shift
    next if @imported.member?(fn)
    fn_task = lookup(fn) and fn_task.invoke
    ext = File.extname(fn)
    loader = @loaders[ext] || @default_loader
    loader.load(fn)
    if fn_task = lookup(fn) and fn_task.needed?
      fn_task.reenable
      fn_task.invoke
      loader.load(fn)
    end
    @imported << fn
  end
end

#load_rakefileObject

Find the rakefile and then load it and any pending imports.



125
126
127
128
129
# File 'lib/rake/application.rb', line 125

def load_rakefile
  standard_exception_handling do
    raw_load_rakefile
  end
end

#optionsObject

Application options from the command line



168
169
170
# File 'lib/rake/application.rb', line 168

def options
  @options ||= Options.new
end

#parse_task_string(string) ⇒ Object

:nodoc:



186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
# File 'lib/rake/application.rb', line 186

def parse_task_string(string) # :nodoc:
  /^([^\[]+)(?:\[(.*)\])$/ =~ string.to_s

  name           = $1
  remaining_args = $2

  return string, [] unless name
  return name,   [] if     remaining_args.empty?

  args = []

  begin
    /\s*((?:[^\\,]|\\.)*?)\s*(?:,\s*(.*))?$/ =~ remaining_args

    remaining_args = $2
    args << $1.gsub(/\\(.)/, '\1')
  end while remaining_args

  return name, args
end

:nodoc:



715
716
717
718
# File 'lib/rake/application.rb', line 715

def print_rakefile_directory(location) # :nodoc:
  $stderr.puts "(in #{Dir.pwd})" unless
    options.silent or original_dir == location
end

#rake_require(file_name, paths = $LOAD_PATH, loaded = $LOADED_FEATURES) ⇒ Object

Similar to the regular Ruby require command, but will check for *.rake files in addition to *.rb files.



689
690
691
692
693
694
695
696
697
698
699
700
701
# File 'lib/rake/application.rb', line 689

def rake_require(file_name, paths=$LOAD_PATH, loaded=$LOADED_FEATURES) # :nodoc:
  fn = file_name + ".rake"
  return false if loaded.include?(fn)
  paths.each do |path|
    full_path = File.join(path, fn)
    if File.exist?(full_path)
      Rake.load_rakefile(full_path)
      loaded << fn
      return true
    end
  end
  fail LoadError, "Can't find #{file_name}"
end

#rakefile_location(backtrace = caller) ⇒ Object

:nodoc:



814
815
816
817
818
819
820
821
# File 'lib/rake/application.rb', line 814

def rakefile_location(backtrace=caller) # :nodoc:
  backtrace.map { |t| t[/([^:]+):/, 1] }

  re = /^#{@rakefile}$/
  re = /#{re.source}/i if windows?

  backtrace.find { |str| str =~ re } || ""
end

#raw_load_rakefileObject

:nodoc:



720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
# File 'lib/rake/application.rb', line 720

def raw_load_rakefile # :nodoc:
  rakefile, location = find_rakefile_location
  if (!options.ignore_system) &&
      (options.load_system || rakefile.nil?) &&
      system_dir && File.directory?(system_dir)
    print_rakefile_directory(location)
    glob("#{system_dir}/*.rake") do |name|
      add_import name
    end
  else
    fail "No Rakefile found (looking for: #{@rakefiles.join(', ')})" if
      rakefile.nil?
    @rakefile = rakefile
    Dir.chdir(location)
    print_rakefile_directory(location)
    Rake.load_rakefile(File.expand_path(@rakefile)) if
      @rakefile && @rakefile != ""
    options.rakelib.each do |rlib|
      glob("#{rlib}/*.rake") do |name|
        add_import name
      end
    end
  end
  load_imports
end

#run(argv = ARGV) ⇒ Object

Run the Rake application. The run method performs the following three steps:

  • Initialize the command line options (init).

  • Define the tasks (load_rakefile).

  • Run the top level tasks (top_level).

If you wish to build a custom rake command, you should call init on your application. Then define any tasks. Finally, call top_level to run your top level tasks.



80
81
82
83
84
85
86
# File 'lib/rake/application.rb', line 80

def run(argv = ARGV)
  standard_exception_handling do
    init "rake", argv
    load_rakefile
    top_level
  end
end

#run_with_threadsObject

Run the given block with the thread startup and shutdown.



145
146
147
148
149
150
151
152
153
154
155
156
157
158
# File 'lib/rake/application.rb', line 145

def run_with_threads
  thread_pool.gather_history if options.job_stats == :history

  yield
ensure
  thread_pool.join if defined?(@thread_pool)
  if options.job_stats
    stats = thread_pool.statistics
    puts "Maximum active threads: #{stats[:max_active_threads]} + main"
    puts "Total threads in play:  #{stats[:total_threads_in_play]} + main"
  end
  ThreadHistoryDisplay.new(thread_pool.history).show if
    options.job_stats == :history
end

#set_default_optionsObject

:nodoc:



823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
# File 'lib/rake/application.rb', line 823

def set_default_options # :nodoc:
  options.always_multitask           = false
  options.backtrace                  = false
  options.build_all                  = false
  options.dryrun                     = false
  options.ignore_deprecate           = false
  options.ignore_system              = false
  options.job_stats                  = false
  options.load_system                = false
  options.nosearch                   = false
  options.rakelib                    = %w[rakelib]
  options.show_all_tasks             = false
  options.show_prereqs               = false
  options.show_task_pattern          = nil
  options.show_tasks                 = nil
  options.silent                     = false
  options.suppress_backtrace_pattern = nil
  options.thread_pool_size           = Rake.suggested_thread_count
  options.trace                      = false
  options.trace_output               = $stderr
  options.trace_rules                = false
end

#standard_exception_handlingObject

Provide standard exception handling for the given block.



208
209
210
211
212
213
214
215
216
217
218
219
220
# File 'lib/rake/application.rb', line 208

def standard_exception_handling # :nodoc:
  yield
rescue SystemExit
  # Exit silently with current status
  raise
rescue OptionParser::InvalidOption => ex
  $stderr.puts ex.message
  exit(false)
rescue Exception => ex
  # Exit with error message
  display_error_message(ex)
  exit_because_of_exception(ex)
end

#standard_rake_optionsObject

A list of all the standard options used in rake, suitable for passing to OptionParser.



427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
# File 'lib/rake/application.rb', line 427

def standard_rake_options # :nodoc:
  sort_options(
    [
      ["--all", "-A",
        "Show all tasks, even uncommented ones (in combination with -T or -D)",
        lambda { |value|
          options.show_all_tasks = value
        }
      ],
      ["--backtrace=[OUT]",
        "Enable full backtrace.  OUT can be stderr (default) or stdout.",
        lambda { |value|
          options.backtrace = true
          select_trace_output(options, "backtrace", value)
        }
      ],
      ["--build-all", "-B",
       "Build all prerequisites, including those which are up-to-date.",
       lambda { |value|
         options.build_all = true
       }
      ],
      ["--comments",
        "Show commented tasks only",
        lambda { |value|
          options.show_all_tasks = !value
        }
      ],
      ["--describe", "-D [PATTERN]",
        "Describe the tasks (matching optional PATTERN), then exit.",
        lambda { |value|
          select_tasks_to_show(options, :describe, value)
        }
      ],
      ["--directory", "-C [DIRECTORY]",
        "Change to DIRECTORY before doing anything.",
        lambda { |value|
          Dir.chdir value
          @original_dir = Dir.pwd
        }
      ],
      ["--dry-run", "-n",
        "Do a dry run without executing actions.",
        lambda { |value|
          Rake.verbose(true)
          Rake.nowrite(true)
          options.dryrun = true
          options.trace = true
        }
      ],
      ["--execute", "-e CODE",
        "Execute some Ruby code and exit.",
        lambda { |value|
          eval(value)
          exit
        }
      ],
      ["--execute-print", "-p CODE",
        "Execute some Ruby code, print the result, then exit.",
        lambda { |value|
          puts eval(value)
          exit
        }
      ],
      ["--execute-continue",  "-E CODE",
        "Execute some Ruby code, " +
        "then continue with normal task processing.",
        lambda { |value| eval(value) }
      ],
      ["--jobs",  "-j [NUMBER]",
        "Specifies the maximum number of tasks to execute in parallel. " +
        "(default is number of CPU cores + 4)",
        lambda { |value|
          if value.nil? || value == ""
            value = Float::INFINITY
          elsif value =~ /^\d+$/
            value = value.to_i
          else
            value = Rake.suggested_thread_count
          end
          value = 1 if value < 1
          options.thread_pool_size = value - 1
        }
      ],
      ["--job-stats [LEVEL]",
        "Display job statistics. " +
        "LEVEL=history displays a complete job list",
        lambda { |value|
          if value =~ /^history/i
            options.job_stats = :history
          else
            options.job_stats = true
          end
        }
      ],
      ["--libdir", "-I LIBDIR",
        "Include LIBDIR in the search path for required modules.",
        lambda { |value| $:.push(value) }
      ],
      ["--multitask", "-m",
        "Treat all tasks as multitasks.",
        lambda { |value| options.always_multitask = true }
      ],
      ["--no-search", "--nosearch",
        "-N", "Do not search parent directories for the Rakefile.",
        lambda { |value| options.nosearch = true }
      ],
      ["--prereqs", "-P",
        "Display the tasks and dependencies, then exit.",
        lambda { |value| options.show_prereqs = true }
      ],
      ["--quiet", "-q",
        "Do not log messages to standard output.",
        lambda { |value| Rake.verbose(false) }
      ],
      ["--rakefile", "-f [FILENAME]",
        "Use FILENAME as the rakefile to search for.",
        lambda { |value|
          value ||= ""
          @rakefiles.clear
          @rakefiles << value
        }
      ],
      ["--rakelibdir", "--rakelib", "-R RAKELIBDIR",
        "Auto-import any .rake files in RAKELIBDIR. " +
        "(default is 'rakelib')",
        lambda { |value|
          options.rakelib = value.split(File::PATH_SEPARATOR)
        }
      ],
      ["--require", "-r MODULE",
        "Require MODULE before executing rakefile.",
        lambda { |value|
          begin
            require value
          rescue LoadError => ex
            begin
              rake_require value
            rescue LoadError
              raise ex
            end
          end
        }
      ],
      ["--rules",
        "Trace the rules resolution.",
        lambda { |value| options.trace_rules = true }
      ],
      ["--silent", "-s",
        "Like --quiet, but also suppresses the " +
        "'in directory' announcement.",
        lambda { |value|
          Rake.verbose(false)
          options.silent = true
        }
      ],
      ["--suppress-backtrace PATTERN",
        "Suppress backtrace lines matching regexp PATTERN. " +
        "Ignored if --trace is on.",
        lambda { |value|
          options.suppress_backtrace_pattern = Regexp.new(value)
        }
      ],
      ["--system",  "-g",
        "Using system wide (global) rakefiles " +
        "(usually '~/.rake/*.rake').",
        lambda { |value| options.load_system = true }
      ],
      ["--no-system", "--nosystem", "-G",
        "Use standard project Rakefile search paths, " +
        "ignore system wide rakefiles.",
        lambda { |value| options.ignore_system = true }
      ],
      ["--tasks", "-T [PATTERN]",
        "Display the tasks (matching optional PATTERN) " +
        "with descriptions, then exit. " +
        "-AT combination displays all the tasks, including those without descriptions.",
        lambda { |value|
          select_tasks_to_show(options, :tasks, value)
        }
      ],
      ["--trace=[OUT]", "-t",
        "Turn on invoke/execute tracing, enable full backtrace. " +
        "OUT can be stderr (default) or stdout.",
        lambda { |value|
          options.trace = true
          options.backtrace = true
          select_trace_output(options, "trace", value)
          Rake.verbose(true)
        }
      ],
      ["--verbose", "-v",
        "Log message to standard output.",
        lambda { |value| Rake.verbose(true) }
      ],
      ["--version", "-V",
        "Display the program version.",
        lambda { |value|
          puts "rake, version #{Rake::VERSION}"
          exit
        }
      ],
      ["--where", "-W [PATTERN]",
        "Describe the tasks (matching optional PATTERN), then exit.",
        lambda { |value|
          select_tasks_to_show(options, :lines, value)
          options.show_all_tasks = true
        }
      ],
      ["--no-deprecation-warnings", "-X",
        "Disable the deprecation warnings.",
        lambda { |value|
          options.ignore_deprecate = true
        }
      ],
    ])
end

#system_dirObject

The directory path containing the system wide rakefiles.



752
753
754
# File 'lib/rake/application.rb', line 752

def system_dir # :nodoc:
  @system_dir ||= ENV["RAKE_SYSTEM"] || standard_system_dir
end

#terminal_widthObject

:nodoc:



362
363
364
365
366
367
368
369
370
371
# File 'lib/rake/application.rb', line 362

def terminal_width # :nodoc:
  if @terminal_columns.nonzero?
    result = @terminal_columns
  else
    result = unix? ? dynamic_width : 80
  end
  (result < 10) ? 80 : result
rescue
  80
end

#thread_poolObject

Return the thread pool used for multithreaded processing.



173
174
175
# File 'lib/rake/application.rb', line 173

def thread_pool             # :nodoc:
  @thread_pool ||= ThreadPool.new(options.thread_pool_size || Rake.suggested_thread_count-1)
end

#top_levelObject

Run the top level tasks of a Rake application.



132
133
134
135
136
137
138
139
140
141
142
# File 'lib/rake/application.rb', line 132

def top_level
  run_with_threads do
    if options.show_tasks
      display_tasks_and_comments
    elsif options.show_prereqs
      display_prerequisites
    else
      top_level_tasks.each { |task_name| invoke_task(task_name) }
    end
  end
end

#trace(*strings) ⇒ Object

:nodoc:



413
414
415
416
# File 'lib/rake/application.rb', line 413

def trace(*strings) # :nodoc:
  options.trace_output ||= $stderr
  trace_on(options.trace_output, *strings)
end

#truncate(string, width) ⇒ Object

:nodoc:



395
396
397
398
399
400
401
402
403
# File 'lib/rake/application.rb', line 395

def truncate(string, width) # :nodoc:
  if string.nil?
    ""
  elsif string.length <= width
    string
  else
    (string[0, width - 3] || "") + "..."
  end
end

#truncate_output?Boolean

We will truncate output if we are outputting to a TTY or if we’ve been given an explicit column width to honor

Returns:

  • (Boolean)


318
319
320
# File 'lib/rake/application.rb', line 318

def truncate_output? # :nodoc:
  tty_output? || @terminal_columns.nonzero?
end

#tty_output?Boolean

True if we are outputting to TTY, false otherwise

Returns:

  • (Boolean)


312
313
314
# File 'lib/rake/application.rb', line 312

def tty_output? # :nodoc:
  @tty_output
end

#unix?Boolean

:nodoc:

Returns:

  • (Boolean)


386
387
388
389
# File 'lib/rake/application.rb', line 386

def unix? # :nodoc:
  RbConfig::CONFIG["host_os"] =~
    /(aix|darwin|linux|(net|free|open)bsd|cygwin|solaris|irix|hpux)/i
end

#windows?Boolean

:nodoc:

Returns:

  • (Boolean)


391
392
393
# File 'lib/rake/application.rb', line 391

def windows? # :nodoc:
  Win32.windows?
end