Class: Rake::TestTask

Inherits:
TaskLib show all
Defined in:
lib/rake/testtask.rb

Overview

Create a task that runs a set of tests.

Example:

require "rake/testtask"

Rake::TestTask.new do |t|
  t.libs << "test"
  t.test_files = FileList['test/test*.rb']
  t.verbose = true
end

If rake is invoked with a “TEST=filename” command line option, then the list of test files will be overridden to include only the filename specified on the command line. This provides an easy way to run just one test.

If rake is invoked with a “TESTOPTS=options” command line option, then the given options are passed to the test process after a ‘–’. This allows Test::Unit options to be passed to the test suite.

Examples:

rake test                           # run tests normally
rake test TEST=just_one_file.rb     # run just one test file.
rake test TESTOPTS="-v"             # run in verbose mode
rake test TESTOPTS="--runner=fox"   # use the fox test runner

Constant Summary

Constants included from FileUtilsExt

FileUtilsExt::DEFAULT

Constants included from FileUtils

FileUtils::LN_SUPPORTED, FileUtils::RUBY

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from FileUtilsExt

#nowrite, #rake_check_options, #rake_output_message, #when_writing

Methods included from FileUtils

#ruby, #safe_ln, #sh, #split_all

Methods included from Cloneable

#initialize_copy

Constructor Details

#initialize(name = :test) {|_self| ... } ⇒ TestTask

Create a testing task.

Yields:

  • (_self)

Yield Parameters:



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
# File 'lib/rake/testtask.rb', line 86

def initialize(name=:test)
  @name = name
  @libs = ["lib"]
  @pattern = nil
  @options = nil
  @test_files = nil
  @verbose = false
  @warning = true
  @loader = :rake
  @ruby_opts = []
  @description = "Run tests" + (@name == :test ? "" : " for #{@name}")
  @deps = []
  if @name.is_a?(Hash)
    @deps = @name.values.first
    @name = @name.keys.first
  end
  yield self if block_given?
  @pattern = "test/test*.rb" if @pattern.nil? && @test_files.nil?
  define
end

Instance Attribute Details

#depsObject

Task prerequisites.



75
76
77
# File 'lib/rake/testtask.rb', line 75

def deps
  @deps
end

#descriptionObject

Description of the test task. (default is ‘Run tests’)



72
73
74
# File 'lib/rake/testtask.rb', line 72

def description
  @description
end

#libsObject

List of directories added to $LOAD_PATH before running the tests. (default is ‘lib’)



42
43
44
# File 'lib/rake/testtask.rb', line 42

def libs
  @libs
end

#loaderObject

Style of test loader to use. Options are:

  • :rake – Rake provided test loading script (default).

  • :testrb – Ruby provided test loading script.

  • :direct – Load tests using command line loader.



66
67
68
# File 'lib/rake/testtask.rb', line 66

def loader
  @loader
end

#nameObject

Name of test task. (default is :test)



38
39
40
# File 'lib/rake/testtask.rb', line 38

def name
  @name
end

#optionsObject

Test options passed to the test suite. An explicit TESTOPTS=opts on the command line will override this. (default is NONE)



50
51
52
# File 'lib/rake/testtask.rb', line 50

def options
  @options
end

#patternObject

Glob pattern to match test files. (default is ‘test/test*.rb’)



58
59
60
# File 'lib/rake/testtask.rb', line 58

def pattern
  @pattern
end

#ruby_optsObject

Array of command line options to pass to ruby when running test loader.



69
70
71
# File 'lib/rake/testtask.rb', line 69

def ruby_opts
  @ruby_opts
end

#verboseObject

True if verbose test output desired. (default is false)



45
46
47
# File 'lib/rake/testtask.rb', line 45

def verbose
  @verbose
end

#warningObject

Request that the tests be run with the warning flag set. E.g. warning=true implies “ruby -w” used to run the tests. (default is true)



55
56
57
# File 'lib/rake/testtask.rb', line 55

def warning
  @warning
end

Instance Method Details

#defineObject

Create the tasks defined by this task lib.



108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
# File 'lib/rake/testtask.rb', line 108

def define
  desc @description
  task @name => Array(deps) do
    FileUtilsExt.verbose(@verbose) do
      args =
        "#{ruby_opts_string} #{run_code} " +
        "#{file_list_string} #{option_list}"
      ruby args do |ok, status|
        if !ok && status.respond_to?(:signaled?) && status.signaled?
          raise SignalException.new(status.termsig)
        elsif !ok
          status  = "Command failed with status (#{status.exitstatus})"
          details = ": [ruby #{args}]"
          message =
            if Rake.application.options.trace or @verbose
              status + details
            else
              status
            end

          fail message
        end
      end
    end
  end
  self
end

#file_listObject

:nodoc:



160
161
162
163
164
165
166
167
168
169
# File 'lib/rake/testtask.rb', line 160

def file_list # :nodoc:
  if ENV["TEST"]
    FileList[ENV["TEST"].split(",")]
  else
    result = []
    result += @test_files.to_a if @test_files
    result += FileList[@pattern].to_a if @pattern
    result
  end
end

#file_list_stringObject

:nodoc:



156
157
158
# File 'lib/rake/testtask.rb', line 156

def file_list_string # :nodoc:
  file_list.map { |fn| "\"#{fn}\"" }.join(" ")
end

#lib_pathObject

:nodoc:



152
153
154
# File 'lib/rake/testtask.rb', line 152

def lib_path # :nodoc:
  @libs.join(File::PATH_SEPARATOR)
end

#option_listObject

:nodoc:



136
137
138
139
140
141
142
143
# File 'lib/rake/testtask.rb', line 136

def option_list # :nodoc:
  (ENV["TESTOPTS"] ||
    ENV["TESTOPT"] ||
    ENV["TEST_OPTS"] ||
    ENV["TEST_OPT"] ||
    @options ||
    "")
end

#ruby_opts_stringObject

:nodoc:



145
146
147
148
149
150
# File 'lib/rake/testtask.rb', line 145

def ruby_opts_string # :nodoc:
  opts = @ruby_opts.dup
  opts.unshift("-I\"#{lib_path}\"") unless @libs.empty?
  opts.unshift("-w") if @warning
  opts.join(" ")
end

#ruby_versionObject

:nodoc:



171
172
173
# File 'lib/rake/testtask.rb', line 171

def ruby_version # :nodoc:
  RUBY_VERSION
end

#run_codeObject

:nodoc:



175
176
177
178
179
180
181
182
183
184
# File 'lib/rake/testtask.rb', line 175

def run_code # :nodoc:
  case @loader
  when :direct
    "-e \"ARGV.each{|f| require f}\""
  when :testrb
    "-S testrb"
  when :rake
    "#{__dir__}/rake_test_loader.rb"
  end
end

#test_files=(list) ⇒ Object

Explicitly define the list of test files to be included in a test. list is expected to be an array of file names (a FileList is acceptable). If both pattern and test_files are used, then the list of test files is the union of the two.



81
82
83
# File 'lib/rake/testtask.rb', line 81

def test_files=(list)
  @test_files = list
end