Method: Rake::FileTask.scope_name
- Defined in:
- lib/rake/file_task.rb
.scope_name(scope, task_name) ⇒ Object
Apply the scope to the task name according to the rules for this kind of task. File based tasks ignore the scope when creating the name.
53 54 55 |
# File 'lib/rake/file_task.rb', line 53 def scope_name(scope, task_name) Rake.from_pathname(task_name) end |