Method: Rake::ThreadPool#history
- Defined in:
- lib/rake/thread_pool.rb
#history ⇒ Object
Return a array of history events for the thread pool.
History gathering must be enabled to be able to see the events (see #gather_history). Best to call this when the job is complete (i.e. after ThreadPool#join is called).
77 78 79 80 81 |
# File 'lib/rake/thread_pool.rb', line 77 def history # :nodoc: @history_mon.synchronize { @history.dup }. sort_by { |i| i[:time] }. each { |i| i[:time] -= @history_start_time } end |