Method: Rake::Task#set_arg_names
- Defined in:
- lib/rake/task.rb
#set_arg_names(args) ⇒ Object
Set the names of the arguments for this task. args should be an array of symbols, one for each argument name.
348 349 350 |
# File 'lib/rake/task.rb', line 348 def set_arg_names(args) @arg_names = args.map(&:to_sym) end |